import re

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver

WILDCARD_PATTERN = re.compile(r"(\S+[.][*][.]*)+")


class BaseAttributeSolver(BaseSolver):
    operator = ''

    def __init__(self, resource_types, attribute, value):
        super().__init__(SolverType.ATTRIBUTE)
        self.resource_types = resource_types
        self.attribute = attribute
        self.value = value

    def run(self, graph_connector):
        all_vertices_resource_types = [data for _, data in graph_connector.nodes(data=True) if
                                       self.resource_type_pred(data, self.resource_types)]
        passed_vertices = [data for data in all_vertices_resource_types if self.get_operation(vertex=data)]
        failed_vertices = [resource for resource in all_vertices_resource_types if resource not in passed_vertices]
        return passed_vertices, failed_vertices

    def get_operation(self, vertex):
        if re.match(WILDCARD_PATTERN, self.attribute):
            attribute_patterns = self.get_attribute_patterns(self.attribute)
            attribute_matches = [attr for attr in vertex if any(re.match(attribute_pattern, attr) for attribute_pattern in attribute_patterns)]
            if attribute_matches:
                return self.resource_type_pred(vertex, self.resource_types) and any(self._get_operation(vertex=vertex, attribute=attr) for attr in attribute_matches)
        return self.resource_type_pred(vertex, self.resource_types) and self._get_operation(vertex=vertex, attribute=self.attribute)

    def _get_operation(self, vertex, attribute):
        raise NotImplementedError

    @staticmethod
    def get_attribute_patterns(attribute):
        index_pattern = r"[\d+]"
        split_by_dots = attribute.split(".")
        pattern_parts = []
        for i, attr_part in enumerate(split_by_dots):
            if attr_part == "*":
                pattern_parts.append(index_pattern)
            else:
                pattern_parts.append(f"({attr_part})")
        pattern = "[.]".join(pattern_parts)
        pattern_with_index = re.compile(pattern)

        pattern_parts_without_index = []
        for pattern_part in pattern_parts:
            if pattern_part != index_pattern:
                pattern_parts_without_index.append(pattern_part)

        pattern = "[.]".join(pattern_parts_without_index)
        pattern_without_index = re.compile(pattern)

        return pattern_with_index, pattern_without_index



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class DocDBTLS(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DocDB TLS is not disabled"
        id = "CKV_AWS_90"
        supported_resources = ['AWS::DocDB::DBClusterParameterGroup']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'Properties' in conf.keys():
            if 'Parameters' in conf['Properties'].keys():
                if 'tls' in conf['Properties']['Parameters'].keys():
                    if conf['Properties']['Parameters']['tls'] == "disabled":
                        return CheckResult.FAILED
        return CheckResult.PASSED


check = DocDBTLS()



from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class S3Versioning(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure the S3 bucket has versioning enabled"
        id = "CKV_AWS_21"
        supported_resources = ['AWS::S3::Bucket']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/VersioningConfiguration/Status'

    def get_expected_value(self):
        """
        Returns the default expected value, governed by provider best practices
        """

        return 'Enabled'


check = S3Versioning()



from checkov.common.graph.checks_infra.enums import SolverType
from checkov.terraform.checks_infra.solvers.connections_solvers.base_connection_solver import BaseConnectionSolver

from checkov.terraform.graph_builder.graph_components.attribute_names import CustomAttributes


class ComplexConnectionSolver(BaseConnectionSolver):
    def __init__(self, solvers, operator):
        self.solver_type = SolverType.COMPLEX_CONNECTION
        if solvers is None:
            solvers = []
        self.solvers = solvers
        self.operator = operator

        resource_types = []
        connected_resources_types = []
        for sub_solver in self.solvers:
            if sub_solver.solver_type in [SolverType.CONNECTION, SolverType.COMPLEX_CONNECTION]:
                resource_types.extend(sub_solver.resource_types)
                connected_resources_types.extend(sub_solver.connected_resources_types)
            elif sub_solver.solver_type in [SolverType.ATTRIBUTE]:
                resource_types.extend(sub_solver.resource_types)
        resource_types = list(set(resource_types))
        connected_resources_types = list(set(connected_resources_types))

        super().__init__(resource_types, connected_resources_types)

    @staticmethod
    def filter_duplicates(checks):
        return list({check[CustomAttributes.ID]: check for check in checks}.values())

    def filter_results(self, passed: list, failed: list):
        filters = []
        filter_solvers = [sub_solver for sub_solver in self.solvers if sub_solver.solver_type == SolverType.FILTER]
        for sub_solver in filter_solvers:
            filters.append(sub_solver._get_operation())
        if filters:
            for filter_pred in filters:
                passed = list(filter(filter_pred, passed))
                failed = list(filter(filter_pred, failed))
        passed = self.filter_duplicates(passed)
        failed = self.filter_duplicates(failed)
        return passed, failed

    def get_sorted_connection_solvers(self):
        connection_solvers = [sub_solver for sub_solver in self.solvers if
                              sub_solver.solver_type in [SolverType.CONNECTION, SolverType.COMPLEX_CONNECTION]]
        filter_solvers = [sub_solver for sub_solver in self.solvers if sub_solver.solver_type == SolverType.FILTER]

        resource_types_to_filter = []
        for filter_solver in filter_solvers:
            if filter_solver.attribute == 'resource_type':
                resource_types_to_filter.extend(filter_solver.value)

        sorted_connection_solvers = []
        connection_solvers_with_filtered_resource_types = []
        for connection_solver in connection_solvers:
            if any(r in resource_types_to_filter for r in connection_solver.resource_types + connection_solver.connected_resources_types):
                connection_solvers_with_filtered_resource_types.append(connection_solver)
            else:
                sorted_connection_solvers.append(connection_solver)

        sorted_connection_solvers.extend(connection_solvers_with_filtered_resource_types)
        return sorted_connection_solvers

    def run_attribute_solvers(self, graph_connector):
        attribute_solvers = [sub_solver for sub_solver in self.solvers if
                             sub_solver.solver_type in [SolverType.ATTRIBUTE, SolverType.COMPLEX]]
        passed_attributes, failed_attributes = [], []
        for attribute_solver in attribute_solvers:
            passed_solver, failed_solver = attribute_solver.run(graph_connector)
            passed_attributes.extend(passed_solver)
            failed_attributes.extend(failed_solver)
        return passed_attributes, failed_attributes



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ELBUsesSSL(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that Elastic Load Balancer(s) uses SSL certificates provided by AWS Certificate Manager"
        id = "CKV_AWS_127"
        supported_resources = ['aws_elb']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'listener' in conf:
            for listener in conf['listener']:
                if 'ssl_certificate_id' not in listener:
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = ELBUsesSSL()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class DynamoDBTablesEncrypted(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure DynamoDB Tables are encrypted using KMS"
        id = "CKV_AWS_119"
        supported_resources = ["aws_dynamodb_table"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "server_side_encryption/[0]/enabled"


check = DynamoDBTablesEncrypted()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class GKESecureBootforShieldedNodes(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Secure Boot for Shielded GKE Nodes is Enabled"
        id = "CKV_GCP_68"
        supported_resources = ['google_container_cluster', 'google_container_node_pool']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'node_config' in conf.keys():
            node = conf["node_config"][0]
            if isinstance(node, dict) and 'shielded_instance_config' in node:
                monitor = node["shielded_instance_config"][0]
                if monitor["enable_secure_boot"] == [True]:
                    return CheckResult.PASSED
                else:
                    return CheckResult.FAILED
            else:
                # as default is true this is a pass
                return CheckResult.FAILED
        else:
            return CheckResult.UNKNOWN

    def get_inspected_key(self):
        return 'node_config/[0]/shielded_instance_config/[0]/enable_secure_boot'

    def get_expected_values(self):
        return [True]


check = GKESecureBootforShieldedNodes()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DBInstanceLogging(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that respective logs of Amazon Relational Database Service (Amazon RDS) are enabled"
        id = "CKV_AWS_129"
        supported_resources = ["aws_db_instance"]
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        # which log types are chooseable depends on the engine and engine version and even region,
        # therefore it is only checked, if something is enabled.
        if conf.get("enabled_cloudwatch_logs_exports", [[]])[0]:
            return CheckResult.PASSED

        return CheckResult.FAILED


check = DBInstanceLogging()



import json
import logging
import os

import yaml

from checkov.common.graph.checks_infra.base_parser import BaseGraphCheckParser
from checkov.common.graph.checks_infra.registry import BaseRegistry
from checkov.terraform.checks_infra.resources_types import resources_types

CHECKS_POSSIBLE_ENDING = [".yaml", ".yml"]


class Registry(BaseRegistry):
    def __init__(self, parser=BaseGraphCheckParser(), checks_dir=None):
        super().__init__(parser)
        self.checks = []
        self.parser = parser
        self.checks_dir = checks_dir if checks_dir else \
            os.path.join(os.path.dirname(os.path.dirname(__file__)), "checks", "graph_checks")

    def load_checks(self):
        self._load_checks_from_dir(self.checks_dir)

    def _load_checks_from_dir(self, dir: str):
        for root, d_names, f_names in os.walk(dir):
            for file in f_names:
                file_ending = os.path.splitext(file)[1]
                if file_ending in CHECKS_POSSIBLE_ENDING:
                    with open(f'{root}/{file}', "r") as f:
                        if dir != self.checks_dir:
                            # This is a custom check, log its loading
                            logging.info(f"loading {file}")
                        check_yaml = yaml.safe_load(f)
                        check_json = json.loads(json.dumps(check_yaml))
                        check = self.parser.parse_raw_check(check_json, resources_types=self._get_resource_types(check_json))
                        if not any([c for c in self.checks if check.id == c.id]):
                            self.checks.append(check)

    def load_external_checks(self, dir: str):
        self._load_checks_from_dir(dir)

    @staticmethod
    def _get_resource_types(check_json):
        provider = check_json.get("scope", {}).get("provider", "").lower()
        return resources_types.get(provider)



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_float
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class GKELegacyInstanceMetadataDisabled(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure legacy Compute Engine instance metadata APIs are Disabled"
        id = "CKV_GCP_67"
        supported_resources = ['google_container_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
        looks for min_master_version =1.12 which ensures that legacy metadata endpoints are disabled
        https://www.terraform.io/docs/providers/google/r/compute_ssl_policy.html
        :param conf: google_container_cluster configuration
        :return: <CheckResult>
        """
        if 'min_master_version' in conf:
            min_master_version = force_float(conf.get('min_master_version')[0])
            if min_master_version and min_master_version >= 1.12:
                return CheckResult.PASSED

        return CheckResult.FAILED

    def get_inspected_key(self):
        return 'min_master_version'

    def get_expected_value(self):
        return "1.12"


check = GKELegacyInstanceMetadataDisabled()
